"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const querystring = require("querystring");
const controller_1 = require("@curveball/controller");
const http_errors_1 = require("@curveball/http-errors");
const errors_1 = require("../errors");
const oauth2Service = require("../service");
const service_1 = require("../../log/service");
const types_1 = require("../../log/types");
const service_2 = require("../../oauth2-client/service");
const userAppPermissions = require("../../user-app-permissions/service");
const jwt_1 = require("../jwt");
/**
 * The Authorize controller is responsible for handing requests to the oauth2
 * authorize endpoint.
 *
 * There are 2 oauth2 grant types handled here: 'implicit' and
 * 'authorization_code'. Implicit is disabled by default and will probably
 * be removed from a future version.
 */
class AuthorizeController extends controller_1.default {
    async get(ctx) {
        ctx.response.type = 'text/html';
        const params = parseAuthorizationQuery(ctx.query);
        let oauth2Client;
        try {
            oauth2Client = await (0, service_2.findByClientId)(params.clientId);
        }
        catch (e) {
            if (e instanceof http_errors_1.NotFound) {
                throw new errors_1.InvalidClient('Client id incorrect');
            }
            else {
                // Rethrow
                throw e;
            }
        }
        if (!params.redirectUri) {
            throw new errors_1.InvalidRequest('The "redirect_uri" parameter must be provided');
        }
        const grantType = params.responseType === 'token' ? 'implicit' : 'authorization_code';
        if (!oauth2Client.allowedGrantTypes.includes(grantType)) {
            throw new errors_1.UnsupportedGrantType('The current client is not allowed to use the ' + grantType + ' grant_type');
        }
        try {
            await oauth2Service.requireRedirectUri(oauth2Client, params.redirectUri);
        }
        catch (err) {
            await (0, service_1.default)(types_1.EventType.oauth2BadRedirect, ctx);
            throw err;
        }
        if (ctx.session.user === undefined) {
            // Use is not logged in, we need to send them through the login process
            // first. The user will come back here after.
            this.redirectToLogin(ctx, { 'continue': ctx.request.requestTarget });
            return;
        }
        await userAppPermissions.setPermissions(oauth2Client.app, ctx.session.user, params.scope);
        if (params.responseType === 'token') {
            return this.tokenRedirect(ctx, oauth2Client, params);
        }
        else {
            if (oauth2Client.requirePkce && !params.codeChallenge) {
                throw new errors_1.InvalidRequest('This endpoint requires that OAuth2 client support PKCE, and your client did not pass the correct parameters. Either turn off the PKCE requirement for this OAuth2 client, or upgrade to an OAuth2 client library that supports PKCE.');
            }
            return this.codeRedirect(ctx, oauth2Client, params);
        }
    }
    async tokenRedirect(ctx, oauth2Client, params) {
        const token = await oauth2Service.generateTokenImplicit({
            client: oauth2Client,
            scope: params.scope,
            principal: ctx.session.user,
            browserSessionId: ctx.sessionId,
        });
        ctx.status = 302;
        ctx.response.headers.set('Cache-Control', 'no-cache');
        ctx.response.headers.set('Location', params.redirectUri + '#' + querystring.stringify({
            access_token: token.accessToken,
            token_type: token.tokenType,
            expires_in: token.accessTokenExpires - Math.round(Date.now() / 1000),
            state: params.state
        }));
    }
    async codeRedirect(ctx, oauth2Client, params) {
        const code = await oauth2Service.generateAuthorizationCode({
            client: oauth2Client,
            principal: ctx.session.user,
            scope: params.scope,
            codeChallenge: params.codeChallenge,
            codeChallengeMethod: params.codeChallengeMethod,
            browserSessionId: ctx.sessionId,
            nonce: params.nonce,
        });
        const redirectParams = {
            code: code.code,
        };
        if (params.state)
            redirectParams.state = params.state;
        if (params.responseType === 'code id_token') {
            redirectParams.id_token = await (0, jwt_1.generateJWTIDToken)({
                principal: ctx.session.user,
                client: oauth2Client,
                nonce: params.nonce ?? null,
            });
        }
        ctx.status = 302;
        ctx.response.headers.set('Cache-Control', 'no-cache');
        ctx.response.headers.set('Location', params.redirectUri + '?' + querystring.stringify(redirectParams));
    }
    /**
     * Redirects to login screen, if login failed
     */
    async redirectToLogin(ctx, params) {
        ctx.response.status = 302;
        ctx.response.headers.set('Location', '/login?' + querystring.stringify(params));
    }
}
exports.default = new AuthorizeController();
/**
 * The sole goal of this function parse and validate query string parameters.
 */
function parseAuthorizationQuery(query) {
    if (!['token', 'code', 'code id_token'].includes(query.response_type)) {
        throw new errors_1.InvalidRequest('The "response_type" parameter must be provided, and must be "token" or "code"');
    }
    const responseType = query.response_type;
    if (!query.client_id) {
        throw new errors_1.InvalidRequest('The "client_id" parameter must be provided');
    }
    const clientId = query.client_id;
    /**
     * These are all OpenID parameters that we don't support right now. We're
     * throwing an error to make sure we're not incorrectly implementing
     * OpenID Connect while this is still in progress.
     */
    const notSupportedParams = [
        'prompt',
        'max_age',
        'ui_locales',
        'id_token_hint',
        'login_hint',
        'acr_values',
    ];
    for (const param of notSupportedParams) {
        if (param in query) {
            throw new http_errors_1.NotImplemented(`The "${param}" parameter is currently not implemented. Want support for this? Open a ticket.`);
        }
    }
    if (responseType === 'token') {
        return {
            responseType,
            clientId,
            redirectUri: query.redirect_uri ?? undefined,
            state: query.state ?? undefined,
            scope: query.scope ? query.scope.split(' ') : []
        };
    }
    const scope = query.scope ? query.scope.split(' ') : [];
    if (!query.code_challenge && query.code_challenge_method) {
        throw new errors_1.InvalidRequest('The "code_challenge" must be provided');
    }
    let codeChallengeMethod = undefined;
    const codeChallenge = query.code_challenge;
    if (query.code_challenge_method) {
        switch (query.code_challenge_method) {
            case 'S256':
            case 'plain':
                codeChallengeMethod = query.code_challenge_method;
                break;
            default:
                throw new errors_1.InvalidRequest('The "code_challenge_method" must be "plain" or "S256"');
        }
    }
    else {
        codeChallengeMethod = query.code_challenge ? 'plain' : undefined;
    }
    const displayOptions = ['page', 'popup', 'touch', 'wap'];
    const display = query.display && displayOptions.includes(query.display) ?
        query.display : undefined;
    if (query.responseMode && query.responseMode !== 'query') {
        throw new http_errors_1.NotImplemented('The only supported value for "response_mode" is currently "query"');
    }
    return {
        responseType,
        clientId,
        redirectUri: query.redirect_uri ?? undefined,
        state: query.state ?? undefined,
        scope,
        codeChallenge,
        codeChallengeMethod,
        display,
        nonce: query.nonce ?? undefined,
    };
}
//# sourceMappingURL=authorize.js.map