"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lastTokenId = exports.invalidateTokensByBrowserSessionId = exports.getTokenByRefreshToken = exports.getTokenByAccessToken = exports.generateAuthorizationCode = exports.revokeToken = exports.revokeByAccessRefreshToken = exports.generateTokenFromRefreshToken = exports.validatePKCE = exports.generateTokenOneTimeToken = exports.generateTokenDeveloperToken = exports.generateTokenAuthorizationCode = exports.generateTokenPassword = exports.generateTokenClientCredentials = exports.generateTokenImplicit = exports.getActiveTokens = exports.addRedirectUris = exports.requireRedirectUri = exports.validateRedirectUri = exports.getRedirectUris = void 0;
const http_errors_1 = require("@curveball/http-errors");
const crypto = require("crypto");
const database_1 = require("../database");
const server_settings_1 = require("../server-settings");
const service_1 = require("../principal/service");
const errors_1 = require("./errors");
const crypto_1 = require("../crypto");
const jwt_1 = require("./jwt");
const userAppPermissionsService = require("../user-app-permissions/service");
const oauth2TokenFields = [
    'id',
    'oauth2_client_id',
    'access_token',
    'refresh_token',
    'user_id',
    'access_token_expires',
    'refresh_token_expires',
    'created_at',
    'browser_session_id',
    'scope',
    'grant_type',
];
async function getRedirectUris(client) {
    const result = await (0, database_1.query)('SELECT uri FROM oauth2_redirect_uris WHERE oauth2_client_id = ?', [client.id]);
    return result.map((record) => record.uri);
}
exports.getRedirectUris = getRedirectUris;
async function validateRedirectUri(client, redirectUri) {
    const uris = await getRedirectUris(client);
    return uris.includes(redirectUri);
}
exports.validateRedirectUri = validateRedirectUri;
/**
 * Checks if a redirect_uri is permitted for the client.
 *
 * If not, it will emit an InvalidGrant error
 */
async function requireRedirectUri(client, redirectUrl) {
    const uris = await getRedirectUris(client);
    if (uris.length === 0) {
        throw new errors_1.InvalidGrant('No valid redirect_uri was setup for this OAuth2 client_id');
    }
    if (!uris.includes(redirectUrl)) {
        throw new errors_1.InvalidGrant(`Invalid value for redirect_uri. The redirect_uri you passed (${redirectUrl}) was not in the allowed list of redirect_uris`);
    }
}
exports.requireRedirectUri = requireRedirectUri;
async function addRedirectUris(client, redirectUris) {
    const query = 'INSERT INTO oauth2_redirect_uris (oauth2_client_id, uri) VALUES (?, ?)';
    for (const uri of redirectUris) {
        await database_1.default.raw(query, [client.id, uri]);
    }
}
exports.addRedirectUris = addRedirectUris;
async function getActiveTokens(user) {
    const result = await (0, database_1.default)('oauth2_tokens')
        .select(oauth2TokenFields)
        .where('user_id', user.id)
        .andWhere('refresh_token_expires', '>', Math.floor(Date.now() / 1000));
    return result.map(row => tokenRecordToModel(row, user));
}
exports.getActiveTokens = getActiveTokens;
/**
 * Generates a token for the 'implicit' GrantType
 */
function generateTokenImplicit(options) {
    return generateTokenInternal({
        grantType: 'implicit',
        secretUsed: false,
        ...options,
    });
}
exports.generateTokenImplicit = generateTokenImplicit;
/**
 * Generates a token for the 'implicit' GrantType
 */
function generateTokenClientCredentials(options) {
    return generateTokenInternal({
        grantType: 'client_credentials',
        principal: options.client.app,
        secretUsed: true,
        ...options,
    });
}
exports.generateTokenClientCredentials = generateTokenClientCredentials;
/**
 * Generates a token for the 'implicit' GrantType
 */
function generateTokenPassword(options) {
    return generateTokenInternal({
        grantType: 'password',
        ...options,
        secretUsed: true,
    });
}
exports.generateTokenPassword = generateTokenPassword;
/**
 * This function is used for the authorization_code oauth2 flow.
 *
 * In this flow a user first authenticates itself and grants permisssion to
 * the client. After gaining permission the user gets redirected back to the
 * resource owner with a one-time code.
 *
 * The resource owner then exchanges that code for an access and refresh token.
 */
async function generateTokenAuthorizationCode(options) {
    const codeResult = await (0, database_1.default)('oauth2_codes')
        .first()
        .where({ code: options.code });
    if (!codeResult) {
        throw new errors_1.InvalidRequest('The supplied code was not recognized');
    }
    const codeRecord = codeResult;
    const expirySettings = getTokenExpiry();
    await (0, database_1.default)('oauth2_codes')
        .delete()
        .where({ code: codeRecord.code });
    // Delete immediately.
    await database_1.default.raw('DELETE FROM oauth2_codes WHERE id = ?', [codeRecord.id]);
    validatePKCE(options.codeVerifier, codeRecord.code_challenge ?? undefined, codeRecord.code_challenge_method ?? 'S256');
    if (codeRecord.created_at + expirySettings.code < Math.floor(Date.now() / 1000)) {
        throw new errors_1.InvalidRequest('The supplied code has expired');
    }
    if (codeRecord.client_id !== options.client.id) {
        throw new errors_1.UnauthorizedClient('The client_id associated with the token did not match with the authenticated client credentials');
    }
    const principalService = new service_1.PrincipalService('insecure');
    const user = await principalService.findById(codeRecord.principal_id, 'user');
    if (!user.active) {
        throw new Error(`User ${user.href} is not active`);
    }
    const scope = codeRecord.scope?.split(' ') || [];
    const result = await generateTokenInternal({
        grantType: 'authorization_code',
        principal: user,
        scope,
        ...options,
    });
    if (scope.includes('openid')) {
        const idToken = await (0, jwt_1.generateJWTIDToken)({
            client: options.client,
            principal: user,
            nonce: codeRecord.nonce,
        });
        return {
            ...result,
            idToken: idToken,
        };
    }
    else {
        return result;
    }
}
exports.generateTokenAuthorizationCode = generateTokenAuthorizationCode;
/**
 * Generates a token for the 'implicit' GrantType
 */
function generateTokenDeveloperToken(options) {
    const client = {
        id: 0,
        clientId: 'system',
        clientSecret: '',
        allowedGrantTypes: [],
        requirePkce: false,
        href: '/system/client',
        app: {
            id: 0,
            href: '/system',
            externalId: 'system',
            identity: 'https://curveballjs.org/a12nserver',
            createdAt: new Date('2018-11-01T140000Z'),
            modifiedAt: new Date('2022-09-11T020000Z'),
            type: 'app',
            nickname: 'a12n-server system user',
            active: true,
            system: true,
        }
    };
    return generateTokenInternal({
        grantType: 'developer-token',
        ...options,
        secretUsed: false,
        client,
        scope: [],
    });
}
exports.generateTokenDeveloperToken = generateTokenDeveloperToken;
/**
 * Generates a token for the 'implicit' GrantType
 */
function generateTokenOneTimeToken(options) {
    return generateTokenInternal({
        grantType: 'developer-token',
        ...options,
        secretUsed: false,
        scope: [],
    });
}
exports.generateTokenOneTimeToken = generateTokenOneTimeToken;
/**
 * Don't export this function, use or write a more specific function for each grantType
 */
async function generateTokenInternal(options) {
    if (!options.principal.active) {
        throw new Error('Cannot generate token for inactive user');
    }
    const expirySettings = getTokenExpiry();
    const accessTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.accessToken;
    const refreshTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.refreshToken;
    let accessToken;
    if ((0, server_settings_1.getSetting)('jwt.privateKey') !== null && options.client) {
        accessToken = await (0, jwt_1.generateJWTAccessToken)({
            principal: options.principal,
            client: options.client,
            expiry: expirySettings.accessToken,
            scope: options.scope,
        });
    }
    else {
        accessToken = await (0, crypto_1.generateSecretToken)();
    }
    const refreshToken = await (0, crypto_1.generateSecretToken)();
    const record = {
        oauth2_client_id: options.client?.id || 0,
        access_token: accessToken,
        refresh_token: refreshToken,
        user_id: options.principal.id,
        grant_type: grantTypeId(options.grantType, options.secretUsed),
        scope: options.scope?.join(' ') ?? null,
        access_token_expires: accessTokenExpires,
        refresh_token_expires: refreshTokenExpires,
        browser_session_id: options.browserSessionId || null,
        created_at: Math.trunc(Date.now() / 1000),
    };
    await (0, database_1.default)('oauth2_tokens').insert(record);
    if (options.client && options.principal.type === 'user') {
        await userAppPermissionsService.updateLastUse(options.principal, options.client.app);
    }
    return {
        accessToken,
        refreshToken,
        grantType: options.grantType,
        secretUsed: false,
        accessTokenExpires,
        refreshTokenExpires,
        tokenType: 'bearer',
        principal: options.principal,
        clientId: options.client.id,
        scope: options.scope,
    };
}
function validatePKCE(codeVerifier, codeChallenge, codeChallengeMethod) {
    if (!codeVerifier) {
        if (!codeChallenge) {
            // This request was not initiated with PKCE support, so ignore the validation
            return;
        }
        else {
            // The authorization request started with PKCE, but the token request did not follow through
            throw new errors_1.InvalidRequest('The code verifier was not supplied');
        }
    }
    // For the plain method, the derived code and the code verifier are the same
    let derivedCodeChallenge = codeVerifier;
    if (codeChallengeMethod === 'S256') {
        derivedCodeChallenge = crypto.createHash('sha256').update(codeVerifier).digest('base64')
            .replace(/\+/g, '-')
            .replace(/\//g, '_')
            .replace(/=/g, '');
    }
    if (codeChallenge !== derivedCodeChallenge) {
        throw new errors_1.InvalidGrant('The code verifier does not match the code challenge');
    }
}
exports.validatePKCE = validatePKCE;
/**
 * This function is used for the 'refresh_token' grant.
 *
 * By specifying a refresh token, a new access/refresh token pair gets
 * returned. This also expires the old token.
 */
async function generateTokenFromRefreshToken(client, refreshToken) {
    let oldToken;
    try {
        oldToken = await getTokenByRefreshToken(refreshToken);
    }
    catch (err) {
        if (err instanceof http_errors_1.NotFound) {
            throw new errors_1.InvalidGrant('The refresh token was not recognized');
        }
        else {
            throw err;
        }
    }
    if (oldToken.clientId !== client.id) {
        throw new errors_1.UnauthorizedClient('The client_id associated with the refresh did not match with the authenticated client credentials');
    }
    await revokeToken(oldToken);
    return generateTokenInternal({
        grantType: oldToken.grantType,
        principal: oldToken.principal,
        client,
        scope: oldToken.scope,
        secretUsed: oldToken.secretUsed,
    });
}
exports.generateTokenFromRefreshToken = generateTokenFromRefreshToken;
async function revokeByAccessRefreshToken(client, token) {
    let oauth2Token = null;
    try {
        oauth2Token = await getTokenByAccessToken(token);
    }
    catch (err) {
        if (err instanceof http_errors_1.NotFound) {
            // Swallow since it's okay if the token has been revoked previously or is invalid (Section 2.2 of RFC7009)
        }
        else {
            throw err;
        }
    }
    if (!oauth2Token) {
        try {
            oauth2Token = await getTokenByRefreshToken(token);
        }
        catch (err) {
            if (err instanceof http_errors_1.NotFound) {
                // Swallow since it's okay if the token has been revoked previously or is invalid (Section 2.2 of RFC7009)
            }
            else {
                throw err;
            }
        }
    }
    if (!oauth2Token) {
        return;
    }
    if (oauth2Token.clientId !== client.id) {
        // Treat this as an invalid token and don't do anything
        return;
    }
    await revokeToken(oauth2Token);
}
exports.revokeByAccessRefreshToken = revokeByAccessRefreshToken;
/**
 * Removes a token.
 *
 * This function will not throw an error if the token was deleted before.
 */
async function revokeToken(token) {
    const query = 'DELETE FROM oauth2_tokens WHERE access_token = ?';
    await database_1.default.raw(query, [token.accessToken]);
}
exports.revokeToken = revokeToken;
/**
 * This function is used for the authorization_code grant flow.
 *
 * This function creates an code for a user. The code is later exchanged for
 * a oauth2 access token.
 */
async function generateAuthorizationCode(options) {
    const code = await (0, crypto_1.generateSecretToken)();
    await (0, database_1.default)('oauth2_codes')
        .insert({
        client_id: options.client.id,
        principal_id: options.principal.id,
        code: code,
        code_challenge: options.codeChallenge,
        code_challenge_method: options.codeChallengeMethod,
        scope: options.scope?.join(' '),
        browser_session_id: options.browserSessionId,
        created_at: Math.floor(Date.now() / 1000),
        nonce: options.nonce ?? null,
    });
    return {
        code: code
    };
}
exports.generateAuthorizationCode = generateAuthorizationCode;
/**
 * Returns Token information for an existing Access Token.
 *
 * This effectively gives you all information of an access token if you have
 * just the bearer, and allows you to validate if a bearer token is valid.
 *
 * This function will throw NotFound if the token was not recognized.
 */
async function getTokenByAccessToken(accessToken) {
    const result = await (0, database_1.default)('oauth2_tokens')
        .select(oauth2TokenFields)
        .where('access_token', accessToken)
        .andWhere('access_token_expires', '>', Date.now() / 1000);
    if (!result.length) {
        throw new http_errors_1.NotFound('Access token not recognized');
    }
    const row = result[0];
    const principalService = new service_1.PrincipalService('insecure');
    const principal = await principalService.findById(row.user_id);
    if (!principal.active) {
        throw new Error(`Principal ${principal.href} is not active`);
    }
    return tokenRecordToModel(row, principal);
}
exports.getTokenByAccessToken = getTokenByAccessToken;
/**
 * Returns Token information for an existing Refresh Token.
 *
 * This function will throw NotFound if the token was not recognized.
 */
async function getTokenByRefreshToken(refreshToken) {
    const result = await (0, database_1.default)('oauth2_tokens')
        .select(oauth2TokenFields)
        .where('refresh_token', refreshToken)
        .andWhere('refresh_token_expires', '>', Date.now() / 1000);
    if (!result.length) {
        throw new http_errors_1.NotFound('Refresh token not recognized');
    }
    const row = result[0];
    const principalService = new service_1.PrincipalService('insecure');
    const principal = await principalService.findById(row.user_id);
    if (!principal.active) {
        throw new Error(`Principal ${principal.href} is not active`);
    }
    return tokenRecordToModel(row, principal);
}
exports.getTokenByRefreshToken = getTokenByRefreshToken;
/**
 * Removes all tokens that relate to a specific browser session id.
 *
 * This will cause all access tokens and refresh tokens to be invalidated. Generally
 * used when a user logs out.
 *
 * This doesn't remove all tokens for all sessions, but should remove the tokens that
 * relate to the device the user used to log out.
 */
async function invalidateTokensByBrowserSessionId(browserSessionId) {
    await database_1.default.raw('DELETE FROM oauth2_codes WHERE browser_session_id = ?', [browserSessionId]);
    await database_1.default.raw('DELETE FROM oauth2_tokens WHERE browser_session_id = ?', [browserSessionId]);
}
exports.invalidateTokensByBrowserSessionId = invalidateTokensByBrowserSessionId;
function getTokenExpiry() {
    return {
        accessToken: (0, server_settings_1.getSetting)('oauth2.accessToken.expiry'),
        refreshToken: (0, server_settings_1.getSetting)('oauth2.refreshToken.expiry'),
        code: (0, server_settings_1.getSetting)('oauth2.code.expiry'),
    };
}
async function lastTokenId() {
    const result = await (0, database_1.default)('oauth2_tokens')
        .first('id')
        .orderBy('id', 'DESC');
    return result?.id || 0;
}
exports.lastTokenId = lastTokenId;
function tokenRecordToModel(token, principal) {
    const [grantType, secretUsed] = grantTypeIdInfo(token.grant_type);
    return {
        accessToken: token.access_token,
        refreshToken: token.refresh_token,
        accessTokenExpires: token.access_token_expires,
        refreshTokenExpires: token.refresh_token_expires,
        tokenType: 'bearer',
        grantType,
        secretUsed,
        scope: token.scope?.split(' ') ?? [],
        principal,
        clientId: token.oauth2_client_id,
    };
}
function grantTypeIdInfo(grantType) {
    switch (grantType) {
        case null: return [null, false];
        case 1: return ['implicit', false];
        case 2: return ['client_credentials', true];
        case 3: return ['password', true];
        case 4: return ['authorization_code', false];
        case 5: return ['authorization_code', true];
        case 6: return ['one-time-token', false];
        case 7: return ['developer-token', false];
        default:
            throw new Error('Unknown grant_type in database');
    }
}
function grantTypeId(grantType, secretUsed) {
    switch (grantType) {
        case null:
            return null;
        case 'implicit':
            return 1;
        case 'client_credentials':
            return 2;
        case 'password':
            return 3;
        case 'authorization_code':
            return secretUsed ? 5 : 4;
        case 'refresh_token':
            throw new Error(`Incorrect grantType for a token: ${grantType}. The token must be stored with the original grant_type`);
        case 'one-time-token':
            return 6;
        case 'developer-token':
            return 7;
    }
}
//# sourceMappingURL=service.js.map