"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathName = exports.getPrincipalStats = exports.isIdentityValid = exports.hasUsers = exports.PrincipalService = void 0;
const service_1 = require("../privilege/service");
const database_1 = require("../database");
const http_errors_1 = require("@curveball/http-errors");
const crypto_1 = require("../crypto");
/**
 * This class provides a wrapper around the principal service APIs.
 *
 * It ensures that the caller (the personal or app that's logged in) has
 * the appropriate privileges to read the associated data.
 */
class PrincipalService {
    /**
     * Set up the principal service.
     *
     * You must pass the privileges for the current user, so we can check if
     * the user is allowed to perform the requests.
     *
     * If the string 'insecure' is passed, no privileges will be checked.
     */
    constructor(privileges) {
        if (privileges === 'insecure') {
            this.privileges = new service_1.LazyPrivilegeBox(privileges);
        }
        else {
            this.privileges = privileges;
        }
    }
    async findAll(type) {
        this.privileges.require('a12n:principals:list');
        const filters = {};
        if (type) {
            filters.type = userTypeToInt(type);
        }
        const result = await (0, database_1.default)('principals')
            .where(filters);
        const principals = [];
        for (const principal of result) {
            principals.push(recordToModel(principal));
        }
        return principals;
    }
    async findByIdentity(identity) {
        this.privileges.require('a12n:principals:list');
        const result = await (0, database_1.default)('principals')
            .where({ identity })
            .first();
        if (!result) {
            throw new http_errors_1.NotFound(`Principal with identity: ${identity} not found`);
        }
        return recordToModel(result);
    }
    async save(principal) {
        if (!isExistingPrincipal(principal)) {
            this.privileges.require('a12n:principals:create');
            const externalId = await (0, crypto_1.generatePublicId)();
            const newPrincipalsRecord = {
                identity: principal.identity,
                external_id: externalId,
                nickname: principal.nickname,
                type: userTypeToInt(principal.type),
                active: principal.active ? 1 : 0,
                modified_at: Date.now(),
                created_at: Date.now(),
                system: 0,
            };
            const result = await (0, database_1.insertAndGetId)('principals', newPrincipalsRecord);
            return {
                id: result,
                href: `/${principal.type}/${externalId}`,
                externalId,
                system: false,
                ...principal,
            };
        }
        else {
            // Update user
            this.privileges.require('a12n:principals:update', principal.href);
            if (!isIdentityValid(principal.identity)) {
                throw new http_errors_1.UnprocessableEntity('Identity must be a valid URI');
            }
            principal.modifiedAt = new Date();
            const updatePrincipalsRecord = {
                identity: principal.identity,
                nickname: principal.nickname,
                active: principal.active ? 1 : 0,
                modified_at: principal.modifiedAt.getTime(),
            };
            await (0, database_1.default)('principals')
                .where('id', principal.id)
                .update(updatePrincipalsRecord);
            return principal;
        }
    }
    async findByExternalId(externalId, type) {
        let result = await (0, database_1.default)('principals')
            .select()
            .where({ external_id: externalId })
            .first();
        if (!result && +(externalId) > 0) {
            // Trying to find the principal but now use the id field
            result = await (0, database_1.default)('principals')
                .select()
                .where({ id: +externalId })
                .first();
            if (!result) {
                throw new http_errors_1.NotFound(`Principal with id: ${externalId} not found`);
            }
        }
        const principal = recordToModel(result);
        if (!this.privileges.isPrincipal(principal))
            this.privileges.require('a12n:principals:list');
        if (type && principal.type !== type) {
            throw new http_errors_1.NotFound(`Principal with id ${externalId} does not have type ${type}`);
        }
        return principal;
    }
    /**
     * Finds a principal by its href'.
     *
     * This can be a string like /user/1, or a full url.
     * It can also be the uri listed in the 'identity' field.
     */
    async findByHref(href) {
        this.privileges.require('a12n:principals:list');
        const pathName = getPathName(href);
        const matches = pathName.match(/^\/(user|app|group)\/([0-9a-zA-Z_-]+)$/);
        if (!matches) {
            return this.findByIdentity(href);
        }
        let typeFilter;
        switch (matches[1]) {
            case 'user':
                // Backwards compatibility
                typeFilter = [1, 2, 3];
                break;
            case 'app':
                typeFilter = [2];
                break;
            case 'group':
                // Backwards compatibility
                typeFilter = [3];
                break;
        }
        const result = await (0, database_1.default)('principals')
            .select()
            .whereIn('type', typeFilter)
            .andWhere({ external_id: matches[2] })
            .first();
        if (!result) {
            throw new http_errors_1.NotFound('Principal with href: ' + href + ' not found');
        }
        return recordToModel(result);
    }
    /**
     * Find multiple principals.
     *
     * This function returns the principals as a map, index by their id.
     * This is a helper function typically used by other services to find large numbers
     * of joined principals from other tables fast.
     *
     * If any of the ids in the list is duplicated, they are de-duplicated here.
     * if any of the provided ids are not found, this function will error.
     */
    async findMany(ids) {
        this.privileges.require('a12n:principals:list');
        const records = await (0, database_1.default)('principals')
            .select()
            .whereIn('id', ids);
        const result = new Map(records.map(record => [record.id, recordToModel(record)]));
        for (const id of ids) {
            if (!result.has(id)) {
                throw new http_errors_1.NotFound(`Principal with ${id} not found`);
            }
        }
        return result;
    }
    async findById(id, type) {
        this.privileges.require('a12n:principals:list');
        const result = await (0, database_1.default)('principals')
            .select()
            .where({ id });
        if (result.length !== 1) {
            throw new http_errors_1.NotFound(`Principal with id: ${id} not found`);
        }
        const principal = recordToModel(result[0]);
        if (type && principal.type !== type) {
            throw new http_errors_1.NotFound(`Principal with id ${id} does not have type ${type}`);
        }
        return principal;
    }
    /**
     * Returns the list of members of a group
     */
    async findMembers(group) {
        this.privileges.require('a12n:principals:list');
        const result = await (0, database_1.default)('principals')
            .select('principals.*')
            .innerJoin('group_members', { 'principals.id': 'group_members.user_id' })
            .where({ group_id: group.id })
            .orderBy('nickname');
        const models = [];
        for (const record of result) {
            const model = recordToModel(record);
            models.push(model);
        }
        return models;
    }
    async addMember(group, user) {
        this.privileges.require('admin');
        await (0, database_1.default)('group_members').insert({
            group_id: group.id,
            user_id: user.id
        });
    }
    async replaceMembers(group, users) {
        this.privileges.require('admin');
        await database_1.default.transaction(async (trx) => {
            await trx('group_members')
                .delete()
                .where({
                group_id: group.id
            });
            for (const user of users) {
                await trx('groupmembers')
                    .insert({
                    group_id: group.id,
                    user_id: user.id
                });
            }
            await trx.commit();
        });
    }
    async removeMember(group, user) {
        this.privileges.require('admin');
        await (0, database_1.default)('group_members')
            .delete()
            .where({
            group_id: group.id,
            user_id: user.id,
        });
    }
    /**
     * Returns a list of groups for which the principal is a member
     */
    async findGroupsForPrincipal(principal) {
        this.privileges.require('a12n:principals:list');
        const result = await (0, database_1.default)('principals')
            .select('principals.*')
            .innerJoin('group_members', { 'principals.id': 'group_members.group_id' })
            .where({ user_id: principal.id })
            .orderBy('nickname');
        const models = [];
        for (const record of result) {
            const model = recordToModel(record);
            models.push(model);
        }
        return models;
    }
}
exports.PrincipalService = PrincipalService;
/**
 * Returns true if more than 1 principal exists in the system.
 *
 * If there are 0 principals, this puts a12nserver in setup mode, which allows a
 * person to create the first user in the system, automatically activate it
 * and make them an admin.
 */
async function hasUsers() {
    const result = await (0, database_1.default)('principals')
        .select('id')
        .where({ type: 1 })
        .first();
    return !!result;
}
exports.hasUsers = hasUsers;
function isIdentityValid(identity) {
    const regex = /^(?:[A-Za-z]+:\S*$)?/;
    return regex.test(identity);
}
exports.isIdentityValid = isIdentityValid;
async function getPrincipalStats() {
    const result = await (0, database_1.default)('principals')
        .select(['type', database_1.default.raw('COUNT(*) as total')])
        .groupBy('type');
    const principalStats = {
        user: 0,
        app: 0,
        group: 0
    };
    for (const principal of result) {
        principalStats[userTypeIntToUserType(principal.type)] = principal.total;
    }
    return principalStats;
}
exports.getPrincipalStats = getPrincipalStats;
function recordToModel(user) {
    return {
        id: user.id,
        href: `/${userTypeIntToUserType(user.type)}/${user.external_id}`,
        identity: user.identity,
        externalId: user.external_id,
        nickname: user.nickname,
        createdAt: new Date(user.created_at),
        modifiedAt: new Date(user.modified_at),
        type: userTypeIntToUserType(user.type),
        active: !!user.active,
        system: !!user.system,
    };
}
function isExistingPrincipal(user) {
    return user.id !== undefined;
}
function userTypeToInt(input) {
    switch (input) {
        case 'user': return 1;
        case 'app': return 2;
        case 'group': return 3;
    }
}
function userTypeIntToUserType(input) {
    switch (input) {
        case 1: return 'user';
        case 2: return 'app';
        case 3: return 'group';
        default:
            throw new Error('Unknown user type id: ' + input);
    }
}
/**
 * @private Only exported for testing purposes.
 */
function getPathName(href) {
    let url;
    try {
        url = new URL(href);
    }
    catch {
        return href;
    }
    return url.pathname;
}
exports.getPathName = getPathName;
//# sourceMappingURL=service.js.map