"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasTotp = exports.validateTotp = exports.hasPassword = exports.validatePassword = exports.updatePassword = exports.createPassword = void 0;
const bcrypt = require("bcrypt");
const otplib = require("otplib");
const database_1 = require("../database");
const http_errors_1 = require("@curveball/http-errors");
async function createPassword(user, password) {
    assertValidPassword(password);
    await (0, database_1.default)('user_passwords').insert({
        user_id: user.id,
        password: await bcrypt.hash(password, 12)
    });
}
exports.createPassword = createPassword;
async function updatePassword(user, password) {
    assertValidPassword(password);
    await (0, database_1.default)('user_passwords').insert({
        user_id: user.id,
        password: await bcrypt.hash(password, 12)
    })
        .onConflict('user_id')
        .merge();
}
exports.updatePassword = updatePassword;
/**
 * Returns true or false if the password was correct.
 *
 * Calling this method multiple times might result in a block.
 */
async function validatePassword(user, password) {
    const result = await (0, database_1.default)('user_passwords')
        .select('password')
        .where('user_id', user.id);
    const hashes = result.map(row => row.password);
    for (const hash of hashes) {
        if (await bcrypt.compare(password, hash)) {
            return true;
        }
    }
    return false;
}
exports.validatePassword = validatePassword;
async function hasPassword(user) {
    const result = await (0, database_1.default)('user_passwords')
        .select('user_id')
        .where('user_id', user.id);
    return result.length > 0;
}
exports.hasPassword = hasPassword;
/**
 * Returns true or false if the totp token was correct.
 *
 * Calling this method multiple times might result in a block.
 */
async function validateTotp(user, token) {
    const result = await (0, database_1.default)('user_totp')
        .select('secret')
        .where('user_id', user.id);
    if (!result.length) {
        // Not set up
        return false;
    }
    const secret = result[0].secret;
    return otplib.authenticator.check(token, secret);
}
exports.validateTotp = validateTotp;
/**
 * Returns true or false if the totp was provided or not.
 *
 *
 */
async function hasTotp(user) {
    const result = await (0, database_1.default)('user_totp')
        .select('secret')
        .where('user_id', user.id);
    return result.length !== 0;
}
exports.hasTotp = hasTotp;
function assertValidPassword(password) {
    if (password.length < 8) {
        throw new http_errors_1.UnprocessableEntity('Passwords must be at least 8 characters');
    }
}
//# sourceMappingURL=service.js.map